/*
** pork_color.c - color functions.
** Copyright (C) 2002-2003 Amber Adams <amber@ojnk.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <ncurses.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>

#include <pork_missing.h>
#include <pork_util.h>
#include <pork_list.h>
#include <pork_input.h>
#include <pork_color.h>

// Initialize the colors that we're going to be using.

void init_colors(void) {
	int i;
	int n = 1;

	start_color();

	for (i = 0 ; i < 8 ; i++) {
		int j;

		for (j = 0 ; j < 8 ; j++)
			init_pair(n++, j, i);
	}
}

// Sometimes all the functionality get_cattrs offers just makes things messy.
// This is a simpler version.

attr_t r_cattr(const char *code) {
	attr_t ret_attr;

	get_cattrs(code, COLOR_CODE_MAXLEN, &ret_attr, NULL);
	return (ret_attr);
}

// Takes a color code string and makes it into attributes which are stored
// where attr points (or discarded if it's NULL).  The basic form of the
// color code is <foreground>[,<background>], where foreground and background
// are one of the following:
//
// d = black, D = bold black
// r = red, R = bold red
// g = green, G = bold green
// y = yellow, Y = bold yellow
// b = blue, B = bold blue
// p = purple, P = bold purple
// c = cyan, C = bold cyan
// w = white, W = bold white
//
// You can't have bold colors as background colors.  Says the terminal, so
// don't blame me.

int get_cattrs(const char *code, size_t len, attr_t *attr, u_int32_t *pos) {
	int fgcol;
	int bold;
	int bgcol = 0;
	u_int32_t i;
	attr_t new_attr = 0;

	if (pos != NULL)
		i = *pos;
	else
		i = 0;

	if (code[i] == 'x') {
		if (attr)
			*attr = 0;
		return (0);
	}

	fgcol = c2c(code[i]);
	if (fgcol == -1)
		return (-1);

	bold = isupper(code[i]);

	if (++i < len && code[i] == ',') {
		if (++i < len)
			bgcol = c2c(code[i]);
	} else
		--i;

	if (bgcol == -1) {
		bgcol = 0;
		--i;
	}

	new_attr = COLOR_PAIR((bgcol * 8 + fgcol) + 1);
	if (bold)
		new_attr |= A_BOLD;

	if (pos != NULL)
		*pos = i;

	if (attr != NULL)
		*attr = new_attr;

	return (0);
}

// Code to Color

inline int c2c(char c) {
	switch (tolower(c)) {
		case 'd':
			return (0);

		case 'r':
			return (1);

		case 'g':
			return (2);

		case 'y':
			return (3);

		case 'b':
			return (4);

		// Magenta or Purple
		case 'm':
			return (5);

		case 'p':
			return (5);

		case 'c':
			return (6);

		case 'w':
			return (7);
	}

	return (-1);
}
